package Themes;
#
# phroggy.com 5.05
#
# (C) 2004 Andy Lyttle, all rights reserved.
# This code may be distributed and modified under the terms of
# the GNU General Public License.
# http://phroggy.com/opensource/
# 
# Everything to do with themes, backgrounds, and layouts.
#

BEGIN {
  use base 'Exporter';
  our @EXPORT=qw(
    ChooseTheme
    GetThemesCache
    GetBGCache
  );
}

use strict;
use warnings;
use File::Basename;
use Paths;
use Browser;
use PhroggyIO;
use Theme;
use Background;
use Localize;

our($ThemeError,$BGError,%ThemesCache,%BGCache);


sub ChooseTheme {
  # Decide which theme and background to use.  If a theme was
  # specified in the query string, that gets first priority.  Next
  # is the cookie, and finally it picks something based on the
  # user's browser and platform.

  my $ok=0;
  if($Browser::Query{'theme'}) {
    my $test=new Theme($Browser::Query{'theme'});
    my $error=$test->Error();
    if($error) {
      $ThemeError=$error;
    } else {
      $main::Theme=$test;
      _ChooseBackground();
      if($main::Background) {
        $ok=1;
      } else {
        $ThemeError='Default background invalid';
      }
    }
  }
  if($ok==0 && $Browser::Cookies{'theme'}) {
    my $test=new Theme($Browser::Cookies{'theme'});
    my $error=$test->Error();
    if($error) {
      # ignore errors with the cookie theme
    } else {
      $main::Theme=$test;
      _ChooseBackground();
      if($main::Background) {
        $ok=1;
      } # else the default bg for the cookie theme is invalid; revert to default theme
    }
  }
  if($ok==0) {
    my $test=new Theme(_GetDefaultTheme()); # FIXME - GetDefaultTheme should maybe return a theme object?
    my $error=$test->Error();
    if($error) {
      # default theme for this browser is invalid
      Abort("Unable to load themes.");
    } else {
      $main::Theme=$test;
      _ChooseBackground();
      if($main::Background) {
        $ok=1;
      } else {
        # default bg for default theme is invalid
        Abort("Unable to load backgrounds.");
      }
    }
  }
  my $error=ThemeError($ThemeError,$BGError,$main::Theme->Title,$main::Background->Title);
  Alert($error) if($error);
}

sub _GetDefaultTheme {
  # Returns:  default theme name

  # Determine default theme based on browser.
  
  my $useragent=$ENV{"HTTP_USER_AGENT"};

  # For 2.x browsers, default to Simple theme
  my $default="simple";
  
  # For 3.x browsers, default to Plain theme ### DON'T DO THIS ANYMORE due to CSS
#  if($Browser::BrowserLevel>2) {
#    $default="plain" if(_ValidTheme('plain'));
#  }

  # For 4.x browsers, default to Classic theme, and try to detect user's platform
  if($Browser::BrowserLevel>3) {
    $default="classic" if(_ValidTheme('classic'));
    $default="chicago" if($useragent=~/Win/i && _ValidTheme('chicago'));
    $default="luna" if($useragent=~/Windows NT 5\.[^0]/ && _ValidTheme('luna'));
    $default="platinum" if($useragent=~/Mac/ && _ValidTheme('platinum'));
    $default="aqua" if($Browser::BrowserLevel>4 && $useragent=~/Mac OS X/ && _ValidTheme('aqua'));
    $default="aqua" if($Browser::BrowserLevel>4 && $default eq 'platinum' && $useragent=~/OmniWeb/ && _ValidTheme('aqua'));
    $default="bluecurve" if($useragent=~/X11/ && _ValidTheme('bluecurve'));
  }
  return $default;
}

sub _ValidTheme {
  # Arguments:  theme name
  # Returns:  1 if valid, 0 if invalid
  
  my($check)=@_;
  my $theme=new Theme($check);
  if($theme->Error()) {
    return 0;
  } else {
    return 1;
  }
}

sub _ChooseBackground {
  # Sets $main::Background to a background object, or undef if none are valid for the current theme

  # try the background specified in the query string, if any
  if($Browser::Query{'background'}) { # try the background from the query string first
    my $bg=new Background($Browser::Query{'background'});
    if($bg->Valid()) {
      if($bg->OKForTheme($main::Theme)) {
        $main::Background=$bg;
      } else {
        $BGError="Exclusive background";
      }
    } else {
      $BGError="Bad background";
    }
  }
  unless($main::Background) {
    # try the background from the cookie, if we also used the theme from the cookie
    if($Browser::Cookies{'background'} && $Browser::Cookies{'theme'} eq $main::Theme->Name) {
      my $bg=new Background($Browser::Cookies{'background'});
      if($bg->OKForTheme($main::Theme)) {
        $main::Background=$bg;
      }  
    }
  }
  unless($main::Background) { # finally fall back on the default bg for this theme
    my $bg=new Background($main::Theme->Info('defaultbg'));
    if($bg->OKForTheme($main::Theme)) {
      $main::Background=$bg;
    }  
  }
}

sub GetThemesCache {
  # Arguments: force refresh?
  # Returns: hash
  
  my($force)=@_;
  unless(%ThemesCache) { # skip if already loaded
    my $cacheage=0; # if no cache file exists, always create one
    if(-f "$Paths::File{'data'}/themescache.data") {
      $cacheage=(stat "$Paths::File{'data'}/themescache.data")[9];
    }
    my $dirage=(stat $Paths::File{'themes'})[9];
    if($dirage>$cacheage || $force) { # cache is out of date
      _BuildThemesCache();
    } else { # cache is up to date
      _ReadThemesCache();
    }
  }
  return %ThemesCache;
}

sub _BuildThemesCache {
  my @files=glob("$Paths::File{'themes'}/*");
  if(@files<3) {
    Abort('Too few themes are present.  Refusing to build themes cache.');
  }
  open(DB,">$Paths::File{'data'}/themescache.data") or Abort('Cannot write to themescache.data');
  %ThemesCache=();
  foreach my $path (@files) {
    my $theme=basename($path);
    next if($theme=~/[^a-z0-9\-]/);
    my $t=new Theme($theme);
    next if($t->Error);
    next if($t->Info('disabled'));
    my @list=($t->Info('defaultbg'),($t->Info('exclusivebg')?'true':'false'),$t->GetTitleList);
    $ThemesCache{$theme}=\@list;
    print DB join("\t",($theme,@list)),"\n";
  }
  close DB;
}

sub _ReadThemesCache {
  %ThemesCache=();
  my $data=ReadFile("$Paths::File{'data'}/themescache.data",'nocache','notrans');
  foreach my $line (split /\n/,$data) {
    my @list=split /\t/,$line;
    my $theme=shift @list;
    $ThemesCache{$theme}=\@list;
  }  
}

sub GetBGCache {
  # Arguments: force refresh?
  # Returns: hash
  
  my($force)=@_;
  unless(%BGCache) { # skip if already loaded
    my $cacheage=0; # if no cache file exists, always create one
    if(-f "$Paths::File{'data'}/bgcache.data") {
      $cacheage=(stat "$Paths::File{'data'}/bgcache.data")[9];
    }
    my $dirage=(stat $Paths::File{'bg'})[9];
    if($dirage>$cacheage || $force) { # cache is out of date
      _BuildBGCache();
    } else { # cache is up to date
      _ReadBGCache();
    }
  }
  return %BGCache;
}

sub _BuildBGCache {
  my @files=glob("$Paths::File{'bg'}/*.info");
  if(@files<3) {
    Abort('Too few backgrounds are present.  Refusing to build background cache.');
  }
  open(DB,">$Paths::File{'data'}/bgcache.data") or Abort('Cannot write to bgcache.data');
  %BGCache=();
  foreach my $path (@files) {
    my $background=basename($path,'.info');
    next if($background=~/[^a-z0-9\-]/);
    my $b=new Background($background);
    next unless($b->Valid);
    my @list=$b->GetTitleList;
    $BGCache{$background}=\@list;
    print DB join("\t",($background,@list)),"\n";
  }
  close DB;
}

sub _ReadBGCache {
  %BGCache=();
  my $data=ReadFile("$Paths::File{'data'}/bgcache.data",'nocache','notrans');
  foreach my $line (split /\n/,$data) {
    my @list=split /\t/,$line;
    my $background=shift @list;
    $BGCache{$background}=\@list;
  }
}


1;
